/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "gcc-compat.h"
#include "para.h"
#include "list.h"
#include "filter.h"

DEFINE_FILTER_ARRAY(filters);

int filter_init(struct filter *all_filters)
{
	struct filter *f;
	int ret;

	for (f = all_filters; f->name; f++) {
//		PARA_DEBUG_LOG("initializing %s filter\n", f->name);
		ret = f->init(f);
		if (ret < 0)
			return ret;
	}
	return 1;
}

int filter_io(struct filter_chain_info *fci)
{
	struct filter_node *fn;
	char *ib = fci->inbuf;
	size_t *loaded = fci->in_loaded;
	int converted = 0;

	list_for_each_entry(fn, &fci->filters, node) {
		int ret;
		if (*loaded && !fn->loaded) {
			ret = fn->filter->convert(ib, *loaded, fn, fci);
			if (ret < 0)
				return -1;
			*loaded -= ret;
			converted += ret;
			if (*loaded && ret) {
//				PARA_DEBUG_LOG("moving %d bytes in input buffer for %s filter\n",
//					*loaded,  fn->filter->name);
				memmove(ib, ib + ret, *loaded);
			}
		}
		ib = fn->buf;
		loaded = &fn->loaded;
	}
	return converted;
}

/* filter_io.c */
void close_filters(struct filter_chain_info *fci)
{
	struct filter_node *fn;

	list_for_each_entry(fn, &fci->filters, node) {
		PARA_DEBUG_LOG("closing %s filter\n", fn->filter->name);
		fn->filter->close(fn);
	}
}

static struct filter_node *parse_filter_args(int filter_num, char *name,
		char *options)
{
	struct filter *f = &filters[filter_num];
	struct filter_node *fn = para_calloc(sizeof(struct filter_node));
	int i;

	fn->filter = f;

	PARA_DEBUG_LOG("%s, options: %s\n", f->name,
		options? options : "(none)");
	if (options) {
//		PARA_DEBUG_LOG("%s options: %s\n", name, options);
		fn->argc = split_args(options, &fn->argv, ':');
//		PARA_DEBUG_LOG("argc = %d, argv[0]: %s\n", fn->argc, fn->argv[0]);
		for (i = fn->argc; i >= 0; i--)
			fn->argv[i + 1] = fn->argv[i];
		fn->argc += 2;
	} else {
		fn->argc = 1;
		fn->argv = para_malloc(2 * sizeof(char*));
		fn->argv[1] = NULL;
	}
	fn->argv[0] = name;
	if (f->parse_config) {
		int ret = f->parse_config(fn);
		if (ret < 0) {
			free(fn->argv);
			free(fn);
			return NULL;
		}
	}
	return fn;
}

struct filter_node *check_filter_arg(char *fa)
{
	int j;

	for (j = 0; filters[j].name; j++) {
		const char *name = filters[j].name;
		size_t len = strlen(name);
		char c;
//		PARA_DEBUG_LOG("checking %s\n", name);
		if (strlen(fa) < len)
			continue;
		if (strncmp(name, fa, len))
			continue;
		c = fa[len];
		if (c && c != ':')
			continue;
		if (c && !filters[j].parse_config)
			return NULL;
		return parse_filter_args(j, fa, c? fa + len + 1: NULL);
	}
	return NULL;
}
